@@ -36,11 +36,6 @@ struct isDouble<double>{
 #include "Rstream.hpp"
 
 /* Include the Eigen library. */
-#ifndef EIGEN_DEFAULT_DENSE_INDEX_TYPE
-#define EIGEN_DEFAULT_DENSE_INDEX_TYPE int
-#else
-#define EIGEN_DEFAULT_DENSE_INDEX_TYPE_REDEFINED 1
-#endif
 #ifdef TMB_SAFEBOUNDS
 #undef NDEBUG
 #undef eigen_assert
@@ -58,6 +53,9 @@ void eigen_REprintf(const char* x);
 #include <Eigen/Dense>
 #include <Eigen/Sparse>
 
+/* Workaround side effect when -DEIGEN_USE_LAPACKE is set */
+#undef I
+
 /* Include the CppAD library. (Always turn off debug for cppad) */
 #undef NDEBUG
 #define NDEBUG 1
